@@ -337,6 +337,9 @@ void DebloomAlgorithm<span>::execute_aux (
             /** We may have reach the maximum number of items in the partition. */
             if (partition.size() >= partition.getMaxNbItems())
             {
+				//first erase the destination file because BagFile creator does not erase previous file anymore
+				system::impl::System::file().remove (outputUri);
+				
                 /** We exclude the partition content from the critical false positive file. */
                 end_debloom_partition (
                     partition,
@@ -413,14 +416,7 @@ void DebloomAlgorithm<span>::end_debloom_partition (
     partition.clear ();
 }
 
-/*********************************************************************
-** METHOD  :
-** PURPOSE :
-** INPUT   :
-** OUTPUT  :
-** RETURN  :
-** REMARKS :
-*********************************************************************/
+/***** this appears unused due to a "#if 0" around the block of code that calls this function ****/
 template<size_t span>
 IBloom <typename Kmer<span>::Type>* DebloomAlgorithm<span>::createBloom (
     tools::collections::Iterable<Count>* solidIterable,
@@ -648,6 +644,8 @@ float DebloomAlgorithm<span>::getNbBitsPerKmer (size_t kmerSize, DebloomKind deb
         nbitsPerKmer = log (16*kmerSize*(lg2*lg2))/(lg2*lg2);
         break;
     }
+
+    if (nbitsPerKmer == 0) nbitsPerKmer = 1;
     return nbitsPerKmer;
 }
 
