@@ -81,7 +81,7 @@ namespace gatb  {  namespace core  {   namespace kmer  {   namespace impl {
 static const char* progressFormat0 = "DSK: counting kmers                    ";
 static const char* progressFormat1 = "DSK: Pass %d/%d, Step 1: partitioning    ";
 static const char* progressFormat2 = "DSK: Pass %d/%d, Step 2: counting kmers  ";
-static const char* progressFormat4 = "DSK: nb solid kmers found : %-9ld  ";
+static const char* progressFormat4 = "DSK: nb solid kmers: %-9ld  ";
 
 /*********************************************************************
 ** METHOD  :
@@ -212,7 +212,7 @@ IOptionsParser* SortingCountAlgorithm<span>::getOptionsParser (bool mandatory)
     parser->push_back (new OptionOneParam (STR_KMER_ABUNDANCE_MIN_THRESHOLD,"min abundance hard threshold (only used when min abundance is \"auto\")",false, "2"));
     parser->push_back (new OptionOneParam (STR_HISTOGRAM_MAX,     "max number of values in kmers histogram",        false, "10000"));
     parser->push_back (new OptionOneParam (STR_SOLIDITY_KIND,     "way to compute counts of several files (sum, min, max, one, all, custom)",false, "sum"));
-	parser->push_back (new OptionOneParam (STR_SOLIDITY_CUSTOM,   "when solidity-kind is cutom, specifies list of files where kmer must be present",false, ""));
+	parser->push_back (new OptionOneParam (STR_SOLIDITY_CUSTOM,   "when solidity-kind is custom, specifies list of files where kmer must be present",false, ""));
     parser->push_back (new OptionOneParam (STR_MAX_MEMORY,        "max memory (in MBytes)",                         false, "5000"));
     parser->push_back (new OptionOneParam (STR_MAX_DISK,          "max disk   (in MBytes)",                         false, "0"));
     parser->push_back (new OptionOneParam (STR_URI_SOLID_KMERS,   "output file for solid kmers (only when constructing a graph)", false));
@@ -221,8 +221,10 @@ IOptionsParser* SortingCountAlgorithm<span>::getOptionsParser (bool mandatory)
     parser->push_back (new OptionOneParam (STR_URI_OUTPUT_TMP,    "output directory for temporary files",           false, "."));
     parser->push_back (new OptionOneParam (STR_COMPRESS_LEVEL,    "h5 compression level (0:none, 9:best)",          false, "0"));
     parser->push_back (new OptionOneParam (STR_STORAGE_TYPE,      "storage type of kmer counts ('hdf5' or 'file')", false, "hdf5"  ));
+	parser->push_back (new OptionOneParam (STR_HISTO2D,"compute the 2D histogram (with first file = genome, remaining files = reads)",false,"0"));
+	parser->push_back (new OptionOneParam (STR_HISTO,"output the kmer abundance histogram",false,"0"));
 
-    IOptionsParser* devParser = new OptionsParser ("kmer count, algorithmic options");
+    IOptionsParser* devParser = new OptionsParser ("kmer count, advanced performance tweaks");
 
     devParser->push_back (new OptionOneParam (STR_MINIMIZER_TYPE,    "minimizer type (0=lexi, 1=freq)",                false, "0"));
     devParser->push_back (new OptionOneParam (STR_MINIMIZER_SIZE,    "size of a minimizer",                            false, "10"));
@@ -263,6 +265,72 @@ ICountProcessor<span>* SortingCountAlgorithm<span>::getDefaultProcessor (
     tools::storage::impl::Storage*  otherStorage
 )
 {
+	
+	std::string histo2Dstorage_filename;
+	// build histo2D filename
+	int using_histo_2D = params->get(STR_HISTO2D) ? params->getInt(STR_HISTO2D) : 0 ;
+	if(using_histo_2D)
+	{
+		if(params->get(STR_URI_OUTPUT))
+		{
+			std::string uri_input = params->getStr(STR_URI_OUTPUT);
+			histo2Dstorage_filename =  uri_input + ".histo2D";
+		}
+		else
+		if(params->get(STR_URI_INPUT))
+		{
+			std::string uri_input = params->getStr(STR_URI_INPUT);
+			std::string delimiter = ",";
+			std::string firstbankname = uri_input.substr(0, uri_input.find(delimiter));
+			histo2Dstorage_filename =  system::impl::System::file().getBaseName(firstbankname) + ".histo2D";
+		}
+		else if(params->get(STR_URI_FILE))
+		{
+			std::string uri_input = params->getStr(STR_URI_FILE);
+			std::string delimiter = ",";
+			std::string firstbankname = uri_input.substr(0, uri_input.find(delimiter));
+			histo2Dstorage_filename =  system::impl::System::file().getBaseName(firstbankname) + ".histo2D";
+		}
+		else
+		{
+			histo2Dstorage_filename = "histo2D_resultfile";
+		}
+
+	}
+	//histo1D filename
+	std::string histo1Dstorage_filename;
+
+	int using_histo_1D = params->get(STR_HISTO) ? params->getInt(STR_HISTO) : 0 ;
+	if(using_histo_1D)
+	{
+		if(params->get(STR_URI_OUTPUT))
+		{
+			std::string uri_input = params->getStr(STR_URI_OUTPUT);
+			histo1Dstorage_filename =  uri_input + ".histo";
+		}
+		else
+			if(params->get(STR_URI_INPUT))
+			{
+				std::string uri_input = params->getStr(STR_URI_INPUT);
+				std::string delimiter = ",";
+				std::string firstbankname = uri_input.substr(0, uri_input.find(delimiter));
+				histo1Dstorage_filename =  system::impl::System::file().getBaseName(firstbankname) + ".histo";
+			}
+			else if(params->get(STR_URI_FILE))
+			{
+				std::string uri_input = params->getStr(STR_URI_FILE);
+				std::string delimiter = ",";
+				std::string firstbankname = uri_input.substr(0, uri_input.find(delimiter));
+				histo1Dstorage_filename =  system::impl::System::file().getBaseName(firstbankname) + ".histo";
+			}
+			else
+			{
+				histo1Dstorage_filename = "histo_resultfile";
+			}
+		
+	}
+	
+	
     CountProcessor* result = 0;
 
     if (otherStorage == 0)  { otherStorage = dskStorage; }
@@ -279,7 +347,11 @@ ICountProcessor<span>* SortingCountAlgorithm<span>::getDefaultProcessor (
         new CountProcessorHistogram<span> (
             & otherStorage->getGroup("histogram"),
             params->getInt(STR_HISTOGRAM_MAX),
-            params->getInt(STR_KMER_ABUNDANCE_MIN_THRESHOLD)
+            params->getInt(STR_KMER_ABUNDANCE_MIN_THRESHOLD),
+			using_histo_2D,
+			using_histo_1D,
+			histo2Dstorage_filename,
+			histo1Dstorage_filename
         ),
 
         CountProcessorSolidityFactory<span>::create (*params),
@@ -351,6 +423,7 @@ vector<ICountProcessor<span>*> SortingCountAlgorithm<span>::getDefaultProcessorV
     Storage*        otherStorage
 )
 {
+
     vector<ICountProcessor<span>*> result;
 
     ICountProcessor<span>* dskProcessor = getDefaultProcessor (params, dskStorage, otherStorage);
@@ -479,9 +552,37 @@ void SortingCountAlgorithm<span>::configure ()
         setRepartitor (new Repartitor(storage->getGroup("minimizers")));
     }
 
+	
+	//out file name for histo2D
+	string output_histo2Dname  = getInput()->get(STR_URI_OUTPUT) ?
+	getInput()->getStr(STR_URI_OUTPUT) + ".histo2D"   :
+	(getInput()->getStr(STR_URI_OUTPUT_DIR) + "/" + system::impl::System::file().getBaseName (_bank->getIdNb(0))) + ".histo2D";
+	
+	//printf("out histo name %s  \n",output_histo2Dname.c_str());
+
 	/** We check that the processor is ok, otherwise we build one. */
     if (_processors.size() == 0)  { _processors = getDefaultProcessorVector(_config, getInput(), storage, storage);  };
 
+	
+	/** We check if options are compatible with histo2D */
+	int using_histo_2D = getInput()->get(STR_HISTO2D) ? getInput()->getInt(STR_HISTO2D) : 0 ;
+	if(using_histo_2D)
+	{
+
+		getInput()->setStr(STR_SOLIDITY_KIND, "all");
+		_config._solidityKind = KMER_SOLIDITY_ALL;
+	
+		//check number of banks, must be greater than 1
+		int nbanks = _bank->getBanks().size();
+
+		if( nbanks <2 )
+		{
+			fprintf(stderr,"There must be at least 2 input banks when using -histo2D \n");
+			exit(1);
+		}
+		
+	}
+	
     DEBUG (("SortingCountAlgorithm<span>::configure  END  _bank=%p  _config.isComputed=%d  _repartitor=%p  storage=%p\n",
         _bank, _config._isComputed, _repartitor, storage
     ));
@@ -766,12 +867,12 @@ public:
 	
 	
 	/** Destructor. */
-	~FillPartitions ()
+	virtual ~FillPartitions ()
 	{
 		//printf("destruc fillparti _superkmerFiles %p \n",_superkmerFiles);
 		
 		//add to global parti_info
-		_extern_pInfo += _local_pInfo;
+		_extern_pInfo.add_sync(_local_pInfo);
 	}
 	
 private:
@@ -906,12 +1007,12 @@ public:
 
 
     /** Destructor. */
-    ~FillPartitions ()
+    virtual ~FillPartitions ()
     {
 		//printf("destruc fillparti _superkmerFiles %p \n",_superkmerFiles);
 
         //add to global parti_info
-        _extern_pInfo += _local_pInfo;
+        _extern_pInfo.add_sync(_local_pInfo);
     }
 
 private:
@@ -1042,12 +1143,12 @@ private:
 		
 		
 		/** Destructor. */
-		~FillPartitions ()
+		virtual ~FillPartitions ()
 		{
 			//printf("destruc fillparti _superkmerFiles %p \n",_superkmerFiles);
 			
 			//add to global parti_info
-			_extern_pInfo += _local_pInfo;
+			_extern_pInfo.add_sync(_local_pInfo);
 		}
 		
 	private:
@@ -1082,6 +1183,7 @@ void SortingCountAlgorithm<span>::fillPartitions (size_t pass, Iterator<Sequence
 		
 		DEBUG (("SortingCountAlgorithm<span>::fillPartitions  _kmerSize=%d _minim_size=%d \n", _config._kmerSize, _config._minim_size));
 		
+		_nbKmersPerPartitionPerBank.clear();
 		if(_config._solidityKind != KMER_SOLIDITY_SUM)
 		{
 			/** We delete the previous partitions storage. */
@@ -1125,72 +1227,89 @@ void SortingCountAlgorithm<span>::fillPartitions (size_t pass, Iterator<Sequence
 		/** We have to reinit the progress instance since it may have been used by SampleRepart before. */
 		_progress->init();
 		
-		/** We may have several input banks instead of a single one. */
-		std::vector<Iterator<Sequence>*> itBanks =  itSeq->getComposition();
-		
-		/** We first reset the vector holding the kmers number for each partition and for each bank.
-		 * It can be seen as the following matrix:
-		 *
-		 *           part0  part1  part2 ... partJ
-		 *   bank0    xxx    xxx    xxx       xxx
-		 *   bank1    xxx    xxx    xxx       xxx
-		 *    ...
-		 *   bankI    xxx    xxx    xxx       xxx
-		 *
-		 *   Here xxx is the number of items found for the bank I in the partition J
-		 */
-		_nbKmersPerPartitionPerBank.clear();
-		
-		/** We launch the iteration of the sequences iterator with the created functors. */
-		for (size_t i=0; i<itBanks.size(); i++)
-		{
-			size_t groupSize   = 1000;
+		if (_config._solidityKind == KMER_SOLIDITY_SUM) {
+			/** We launch the iteration of the sequences iterator with the created
+			 * functors. */
+
+			size_t groupSize = 1000;
 			bool deleteSynchro = true;
-			
-			/** We fill the partitions. Each thread will read synchronously and will call FillPartitions
-			 * in a synchronous way (in order to have global BanksStats correctly computed). */
-			
-			if(_config._solidityKind == KMER_SOLIDITY_SUM)
-			{
-				getDispatcher()->iterate (itBanks[i], FillPartitions<span,true> (
-																			model, _config._nb_passes, pass, _config._nb_partitions, _config._nb_cached_items_per_core_per_part, _progress, _bankStats, _tmpPartitions, *_repartitor, pInfo,_superKstorage
-																			), groupSize, deleteSynchro);
-			}
-			else
-			{
-				getDispatcher()->iterate (itBanks[i], FillPartitions<span,false> (
-																				 model, _config._nb_passes, pass, _config._nb_partitions, _config._nb_cached_items_per_core_per_part, _progress, _bankStats, _tmpPartitions, *_repartitor, pInfo,_superKstorage
-																				 ), groupSize, deleteSynchro);
-			}
-			
-			
-			/** We flush the partitions in order to be sure to have the exact number of items per partition. */
-			if(_config._solidityKind != KMER_SOLIDITY_SUM)
-			{
-				_tmpPartitions->flush();
-				
-				/** We get a snapshot of items number in each partition. */
-				vector<size_t> nbItems;
-				for (size_t p=0; p<_config._nb_partitions; p++)
-				{
-				 nbItems.push_back ((*_tmpPartitions)[p].getNbItems()); //todo for multi count
-				}
-				
-				/** We add the current number of kmers in each partition for the reached ith bank. */
-				_nbKmersPerPartitionPerBank.push_back (nbItems);
-			}
-			
-			//GR: close the input bank here with call to finalize
-			itBanks[i]->finalize();
-		}
-		
-		if(_config._solidityKind == KMER_SOLIDITY_SUM)
-		{
+
+			/** We fill the partitions. Each thread will read synchronously and will
+			 * call FillPartitions in a synchronous way (in order to have global
+			 * BanksStats correctly computed). */
+			getDispatcher()->iterate(
+				itSeq,
+				FillPartitions<span, true>(
+				    model, _config._nb_passes, pass, _config._nb_partitions,
+				    _config._nb_cached_items_per_core_per_part, _progress, _bankStats,
+				    _tmpPartitions, *_repartitor, pInfo, _superKstorage),
+				groupSize, deleteSynchro);
+
+			// GR: close the input bank here with call to finalize
+			itSeq->finalize();
+
 			_superKstorage->flushFiles();
 			_superKstorage->closeFiles();
+		} else {
+			/** We may have several input banks instead of a single one. */
+			std::vector<Iterator<Sequence>*> itBanks =  itSeq->getComposition();
+
+			/** We first reset the vector holding the kmers number for each partition and for each bank.
+			 * It can be seen as the following matrix:
+			 *
+			 *           part0  part1  part2 ... partJ
+			 *   bank0    xxx    xxx    xxx       xxx
+			 *   bank1    xxx    xxx    xxx       xxx
+			 *    ...
+			 *   bankI    xxx    xxx    xxx       xxx
+			 *
+			 *   Here xxx is the number of items found for the bank I in the partition J
+			 */
+
+			/** We launch the iteration of the sequences iterator with the created functors. */
+			for (size_t i=0; i<itBanks.size(); i++)
+			{
+				size_t groupSize   = 1000;
+				bool deleteSynchro = true;
+
+				/** We fill the partitions. Each thread will read synchronously and will call FillPartitions
+				 * in a synchronous way (in order to have global BanksStats correctly computed). */
+
+			getDispatcher()->iterate(
+				itBanks[i],
+				FillPartitions<span, false>(
+					model, _config._nb_passes, pass, _config._nb_partitions,
+					_config._nb_cached_items_per_core_per_part, _progress, _bankStats,
+					_tmpPartitions, *_repartitor, pInfo, _superKstorage),
+				groupSize, deleteSynchro);
+
+				/** We flush the partitions in order to be sure to have the exact number of items per partition. */
+					_tmpPartitions->flush();
+
+					/** We get a snapshot of items number in each partition. */
+					vector<size_t> nbItems;
+					for (size_t p=0; p<_config._nb_partitions; p++)
+					{
+					 nbItems.push_back ((*_tmpPartitions)[p].getNbItems()); //todo for multi count
+					}
+
+					/** We add the current number of kmers in each partition for the reached ith bank. */
+					_nbKmersPerPartitionPerBank.push_back (nbItems);
+
+				//GR: close the input bank here with call to finalize
+				itBanks[i]->finalize();
+			}
+		}
+
+        // force close partitions and re-open them for reading
+        // may prevent crash in large multi-bank counting instance on Lustre filesystems
+		if(_config._solidityKind != KMER_SOLIDITY_SUM)
+		{
+			string tmpStorageName = getInput()->getStr(STR_URI_OUTPUT_TMP) + "/" + System::file().getTemporaryFilename("dsk_partitions");
+			setPartitions        (0); // close the partitions first, otherwise new files are opened before  closing parti from previous pass
+			setPartitions        ( & (*_tmpPartitionsStorage)().getPartition<Type> ("parts"));
+			
 		}
-		
-		
 	}
 
 /*********************************************************************
@@ -1323,8 +1442,8 @@ void SortingCountAlgorithm<span>::fillSolidKmers_aux (ICountProcessor<span>* pro
 
             /** If we have several input banks, we may have to compute kmer solidity for each bank, which
              * can be currently done only with sorted vector. */
-            bool forceVector  = _nbKmersPerPartitionPerBank.size() > 1 && \
-                                ( _config._solidityKind != KMER_SOLIDITY_SUM);
+			bool forceVector  =   _nbKmersPerPartitionPerBank.size() > 1 && ( _config._solidityKind != KMER_SOLIDITY_SUM);
+
 
             ICommand* cmd = 0;
 
